@@ -16,12 +16,12 @@ from watchmen_storage import ask_disable_compiled_cache, ColumnNameLiteral, Enti
 	EntityId, EntityIdHelper, EntityList, EntityNotFoundException, EntityPager, EntitySort, \
 	EntityStraightAggregateColumn, EntityStraightColumn, EntityStraightValuesFinder, EntityUpdater, \
 	TooManyEntitiesFoundException, TransactionalStorageSPI, UnexpectedStorageException, \
-	UnsupportedStraightColumnException
+	UnsupportedStraightColumnException, EntityLimitedFinder
 from watchmen_utilities import ArrayHelper, is_blank, serialize_to_json
 from .settings import ask_connection_leak_time_in_seconds, ask_detect_connection_leak_enabled, \
 	ask_print_connection_leak_interval
 from .table_defs import find_table
-from .types import SQLAlchemyStatement
+from .types import SQLAlchemyStatement, get_column_type
 
 logger = getLogger(__name__)
 
@@ -80,7 +80,7 @@ class StorageRDS(TransactionalStorageSPI):
 			self.connection.commit()
 		except Exception as e:
 			raise e
-		else:
+		finally:
 			self.close()
 
 	def rollback_and_close(self) -> None:
@@ -282,6 +282,20 @@ class StorageRDS(TransactionalStorageSPI):
 		else:
 			raise TooManyEntitiesFoundException(f'Too many entities found by finder[{helper}].')
 
+	def find_and_lock_by_id_nowait(self, entity_id: EntityId, helper: EntityIdHelper) -> Optional[Entity]:
+		table = self.find_table(helper.name)
+		statement = select(table).with_for_update(nowait=True)
+		statement = self.build_criteria_for_statement([table], statement, [
+			EntityCriteriaExpression(left=ColumnNameLiteral(columnName=helper.idColumnName), right=entity_id)
+		])
+		data = self.connection.execute(statement).mappings().all()
+		if len(data) == 0:
+			return None
+		elif len(data) == 1:
+			return data[0]
+		else:
+			raise TooManyEntitiesFoundException(f'Too many entities found by finder[{helper}].')
+
 	def find_one(self, finder: EntityFinder) -> Optional[Entity]:
 		data = self.find(finder)
 		if len(data) == 0:
@@ -291,6 +305,17 @@ class StorageRDS(TransactionalStorageSPI):
 		else:
 			raise TooManyEntitiesFoundException(f'Too many entities found by finder[{finder}].')
 
+	def find_one_and_lock_nowait(self, finder: EntityFinder) -> Optional[Entity]:
+		table = self.find_table(finder.name)
+		statement = select(table).with_for_update(nowait=True)
+		data = self.find_on_statement_by_finder(table, statement, finder)
+		if len(data) == 0:
+			return None
+		elif len(data) == 1:
+			return data[0]
+		else:
+			raise TooManyEntitiesFoundException(f'Too many entities found by finder[{finder}].')
+
 	def find_on_statement_by_finder(
 			self, table: Table, statement: SQLAlchemyStatement, finder: EntityFinder
 	) -> EntityList:
@@ -304,12 +329,30 @@ class StorageRDS(TransactionalStorageSPI):
 		statement = select(table)
 		return self.find_on_statement_by_finder(table, statement, finder)
 
+	def find_limited(self, finder: EntityLimitedFinder) -> EntityList:
+		table = self.find_table(finder.name)
+		statement = select(table)
+		statement = self.build_criteria_for_statement([table], statement, finder.criteria)
+		statement = self.build_sort_for_statement(statement, finder.sort)
+		statement = self.build_offset_for_statement(statement, finder.limit, 1)
+		results = self.connection.execute(statement).mappings().all()
+		return ArrayHelper(results).map(lambda x: dict(x)).map(finder.shaper.deserialize).to_list()
+
+	def find_for_update_skip_locked(self, finder: EntityLimitedFinder) -> EntityList:
+		table = self.find_table(finder.name)
+		statement = select(table).with_for_update(skip_locked=True)
+		statement = self.build_criteria_for_statement([table], statement, finder.criteria)
+		statement = self.build_sort_for_statement(statement, finder.sort)
+		statement = self.build_offset_for_statement(statement, finder.limit, 1)
+		results = self.connection.execute(statement).mappings().all()
+		return ArrayHelper(results).map(lambda x: dict(x)).map(finder.shaper.deserialize).to_list()
+
 	def find_distinct_values(self, finder: EntityDistinctValuesFinder) -> EntityList:
 		table = self.find_table(finder.name)
 		if len(finder.distinctColumnNames) != 1 or not finder.distinctValueOnSingleColumn:
 			statement = select(*ArrayHelper(finder.distinctColumnNames).map(text).to_list()).select_from(table)
 		else:
-			statement = select(distinct(finder.distinctColumnNames[0])).select_from(table)
+			statement = select(distinct(text(finder.distinctColumnNames[0]))).select_from(table)
 		return self.find_on_statement_by_finder(table, statement, finder)
 
 	# noinspection PyMethodMayBeStatic
@@ -322,15 +365,19 @@ class StorageRDS(TransactionalStorageSPI):
 			if straight_column.arithmetic == EntityColumnAggregateArithmetic.COUNT:
 				return func.count(1).label(self.get_alias_from_straight_column(straight_column))
 			elif straight_column.arithmetic == EntityColumnAggregateArithmetic.SUM:
-				return func.sum(straight_column.columnName).label(self.get_alias_from_straight_column(straight_column))
+				return func.sum(literal_column(straight_column.columnName)) \
+					.label(self.get_alias_from_straight_column(straight_column))
 			elif straight_column.arithmetic == EntityColumnAggregateArithmetic.AVG:
-				return func.avg(straight_column.columnName).label(self.get_alias_from_straight_column(straight_column))
+				return func.avg(literal_column(straight_column.columnName)) \
+					.label(self.get_alias_from_straight_column(straight_column))
 			elif straight_column.arithmetic == EntityColumnAggregateArithmetic.MAX:
-				return func.max(straight_column.columnName).label(self.get_alias_from_straight_column(straight_column))
+				return func.max(literal_column(straight_column.columnName)) \
+					.label(self.get_alias_from_straight_column(straight_column))
 			elif straight_column.arithmetic == EntityColumnAggregateArithmetic.MIN:
-				return func.min(straight_column.columnName).label(self.get_alias_from_straight_column(straight_column))
+				return func.min(literal_column(straight_column.columnName)) \
+					.label(self.get_alias_from_straight_column(straight_column))
 		elif isinstance(straight_column, EntityStraightColumn):
-			return literal_column(straight_column.columnName) \
+			return literal_column(straight_column.columnName, get_column_type(straight_column.columnType)) \
 				.label(self.get_alias_from_straight_column(straight_column))
 
 		raise UnsupportedStraightColumnException(f'Straight column[{straight_column.to_dict()}] is not supported.')
